/**
 * @file ClientConfig.cs
 * 
 * @author Kevin Niehage
 * 
 * @brief File containing runtime configuration.
 */

using System;
using System.Collections.Generic;
using System.Text;

namespace Cubes3D
{
    class ClientConfig
    {
        private bool   FLocalGame    = false;
        private String FMapFile      = "";
        private bool   FRecordReplay = false;

        private int[] FHighscore = null;

        public ClientConfig()
        {
            FHighscore = new int[5] { -1, -1, -1, -1, -1};
        }

        /**
         * @property LocalGame
         * 
         * @author Kevin Niehage
         * 
         * @brief Defines whether the next game will be a local game.
         */
        public bool LocalGame
        {
            get
            {
                return FLocalGame;
            }

            set
            {
                FLocalGame = value;
            }
        }

        /**
         * @property MapFile
         * 
         * @author Kevin Niehage
         * 
         * @brief Defines the file name of the next map.
         */
        public String MapFile
        {
            get
            {
                return FMapFile;
            }

            set
            {
                FMapFile = value;
            }
        }

        /**
         * @property RecordReplay
         * 
         * @author Kevin Niehage
         * 
         * @brief Defines whether a replay shall be recorded.
         */
        public bool RecordReplay
        {
            get
            {
                return FRecordReplay;
            }

            set
            {
                FRecordReplay = value;
            }
        }

        /**
         * @property LocalGame
         * 
         * @author Kevin Niehage
         * 
         * @brief Holds the current highscore.
         */
        public int[] Highscore
        {
            get
            {
                return FHighscore;
            }
        }

        /**
         * @method SetHighscore
         * 
         * @author Kevin Niehage
         * 
         * @param[in] highscore value
         * 
         * @result highscore ranking
         * 
         * @brief Inserts the passed value into the highscore ranking.
         */
        public int SetHighscore(int AValue)
        {
            int LResult = -1;

            int LIndex = 0;
            for (LIndex = 0; LIndex < FHighscore.Length; LIndex++)
            {
                if ((AValue < FHighscore[LIndex]) || (FHighscore[LIndex] <= 0))
                {
                    LResult = LIndex;
                    break;
                }
            }

            if (LResult >= 0)
            {
                for (LIndex = FHighscore.Length - 1; LIndex > LResult; LIndex--)
                {
                    FHighscore[LIndex] = FHighscore[LIndex - 1];
                }
                FHighscore[LResult] = AValue;
            }

            return LResult;
        }
    }
}
