#define Cubes3D_FULLSCREEN

using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Resolution;

using GRV11;

namespace Cubes3D
{
    public partial class MainForm : Form
    {
        // OpenGL graphics control
        private ClientConfig    FClientConfig  = null;
        private ClientHandler   FClientHandler = null;
        private GRV11.GRControl FGameGRControl = null;
        private ServerHandler   FServerHandler = null;

        private byte FClientWait = 0;
        private byte FServerWait = 0;

        private int FDuration = 0;
        private int FHighscore = -1;

#if Cubes3D_FULLSCREEN
        private Point FResolution;
#endif //Cubes3D_FULLSCREEN

        public MainForm()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            FClientConfig = new ClientConfig();

            GameTabPage.SuspendLayout();
            MainTabControl.SuspendLayout();
            SuspendLayout();

            // manually create OpenGL graphics control
            FGameGRControl = new GRV11.GRControl();
            FGameGRControl.Name = "GameGRControl";
            FGameGRControl.TabIndex = 0;
            FGameGRControl.TabStop = true;
            FGameGRControl.Text = "";
            FGameGRControl.BackColor = Color.Black;

            GameTabPage.Controls.Add(FGameGRControl);

            //FServerHandler = new ServerHandler(System.Environment.CurrentDirectory + "\\test.map");
            //FServerHandler.Active = true;

            GameTabPage.ResumeLayout(false);
            MainTabControl.ResumeLayout(false);
            ResumeLayout(false);

            // set first tab to be shown
            //!!! SET TO TitleTabPage
//            MainTabControl.SelectedTab = GameTabPage;
            MainTabControl.SelectedTab = TitleTabPage;
            Cursor.Hide();
            TitleTimer.Enabled = true;

#if Cubes3D_FULLSCREEN
            FormBorderStyle = FormBorderStyle.None;
            Height = Screen.PrimaryScreen.Bounds.Height;
            Left = 0;
            Top = 0;
            Width = Screen.PrimaryScreen.Bounds.Width;
            //WindowState = FormWindowState.Maximized;

            // set screen resolution
            FResolution = new Point(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height);
            Resolution.ScreenResolution.ChangeResolution(1024, 768);

            BringToFront();
#endif //Cubes3D_FULLSCREEN

            MainForm_ResizeEnd(null, null);
            MainTabControl.CausesValidation = false;
        }

        private void MainForm_ResizeEnd(object sender, EventArgs e)
        {
            // get maximal tab positions
            Point lMaxHidden = HiddenTabPage.PointToScreen(new Point(HiddenTabPage.ClientRectangle.Width - 1, HiddenTabPage.ClientRectangle.Height - 1));
            Point lMaxMain = MainTabControl.PointToScreen(new Point(MainTabControl.ClientRectangle.Width - 1, MainTabControl.ClientRectangle.Height - 1));

            // get minimal tab  positions
            Point lMinHidden = HiddenTabPage.PointToScreen(new Point(0, 0));
            Point lMinMain = MainTabControl.PointToScreen(new Point(0, 0));

            // set Tab Control position
            MainTabControl.Left = (lMinMain.X - lMinHidden.X);
            MainTabControl.Top = (lMinMain.Y - lMinHidden.Y);

            // set Tab Control size
            MainTabControl.Height = ClientRectangle.Height + (lMinHidden.Y - lMinMain.Y) +(lMaxMain.Y - lMaxHidden.Y);
            MainTabControl.Width = ClientRectangle.Width + (lMinHidden.X - lMinMain.X) +(lMaxMain.X - lMaxHidden.X);

            /*
            // set Hidden Tab Page size
            // => needed to set Tab Control region correctly
            HiddenTabPage.Height = MainTabControl.ClientRectangle.Height;
            HiddenTabPage.Width = MainTabControl.ClientRectangle.Width;

            // set Tab Control region
            // => region is smaller than control size
            // => control has to be greater than form
            // => region will be as big as the form's client region
            MainTabControl.Region = new Region(new RectangleF(HiddenTabPage.Left, HiddenTabPage.Top,
                                                              HiddenTabPage.Width, HiddenTabPage.Height));
            */

            if (FGameGRControl != null)
            {
                // set OpenGL control size
                FGameGRControl.Height = ClientRectangle.Height;
                FGameGRControl.Left = 0;
                FGameGRControl.Top = 0;
                FGameGRControl.Width = ClientRectangle.Width;
            }
        }

        private void MainForm_Activated(object sender, EventArgs e)
        {
            if (FGameGRControl != null)
            {
                // set focus on reactivate
                FGameGRControl.Focus();
            }
        }

        // manually process keys
        protected override bool ProcessDialogKey(Keys keyData)
        {
            // close on escape
            if (new KeyEventArgs(keyData).KeyCode == Keys.Escape)
            {
                Close();

                return true;
            }

            if (MainTabControl.SelectedTab == TitleTabPage)
            {
                if (new KeyEventArgs(keyData).KeyCode == Keys.Return)
                {
                    MainTabControl.SelectedTab = MenuTabPage;

                    return true;
                }
            }

            if (MainTabControl.SelectedTab == ServerBTabPage)
            {
                if (new KeyEventArgs(keyData).KeyCode == Keys.Return)
                {
                    FServerWait = 60;

                    return true;
                }
            }

            if (MainTabControl.SelectedTab == GameTabPage)
            {
                if ((FGameGRControl != null) && (FClientHandler != null))
                {
                    FClientHandler.KeyDown(FGameGRControl, new KeyEventArgs(keyData));

                    return true;
                }
            }

            return base.ProcessDialogKey(keyData);
        }

        // manually process arrow keys
        private void MainForm_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {

            if (MainTabControl.SelectedTab == GameTabPage)
            {
                if ((FGameGRControl != null) && (FClientHandler != null))
                {
                    FClientHandler.KeyDown(FGameGRControl, new KeyEventArgs(e.KeyData));
                }
            }
        }

        // manually process arrow keys
        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (MainTabControl.SelectedTab == GameTabPage)
            {
                if ((FGameGRControl != null) && (FClientHandler != null))
                {
                    FClientHandler.KeyDown(FGameGRControl, new KeyEventArgs(e.KeyData));
                }
            }
        }

        // manually process arrow keys
        private void MainForm_KeyUp(object sender, KeyEventArgs e)
        {
            if (MainTabControl.SelectedTab == GameTabPage)
            {
                if ((FGameGRControl != null) && (FClientHandler != null))
                {
                    FClientHandler.KeyUp(FGameGRControl, new KeyEventArgs(e.KeyData));
                }
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (FClientHandler != null)
            {
                FClientHandler.Kill = true;
                FClientHandler = null;
            }

            if (FServerHandler != null)
            {
                FServerHandler.Kill = true;
                FServerHandler = null;
            }

#if Cubes3D_FULLSCREEN
            if (FResolution != null)
            {
                Resolution.ScreenResolution.ChangeResolution(FResolution.X, FResolution.Y);
            }
#endif //Cubes3D_FULLSCREEN
        }

        private void ServerWaitTimer_Tick(object sender, EventArgs e)
        {
            if (FServerHandler != null)
            {
                if (MainTabControl.SelectedTab == ServerBTabPage)
                {
                    ServerWaitLabel.Visible = (!(ServerWaitLabel.Visible));

                    if ((FClientHandler.Acknowledged) && FClientConfig.LocalGame)
                    {
                        MainTabControl.SelectedTab = GameTabPage;
                        FServerHandler.GameRunning = true;
                    }
                    else
                    {
                        FServerWait++;
                        if (FServerWait > 60)
                        {
                            MainTabControl.SelectedTab = GameTabPage;
                            FServerHandler.GameRunning = true;
                        }
                    }
                }
            }
        }

        private void MainTabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (MainTabControl.SelectedTab == TitleTabPage)
            {
                Cursor.Hide();

                TitleTimer.Enabled = true;
            }
            else
            {
                TitleTimer.Enabled = false;
            }

            if (MainTabControl.SelectedTab == MenuTabPage)
            {
                Cursor.Show();
            }

            if (MainTabControl.SelectedTab == OptionsTabPage)
            {
                Cursor.Show();

                if (FClientConfig.RecordReplay)
                {
                    ReplaysValueLabel.Text = "on";
                }
                else
                {
                    ReplaysValueLabel.Text = "off";
                }
            }

            if (MainTabControl.SelectedTab == ClientTabPage)
            {
                Cursor.Show();

                // kill old game enginge
                if (FClientHandler != null)
                {
                    FClientHandler.Kill = true;
                    FClientHandler = null;
                }

                // create new game engine
                FClientHandler = new ClientHandler(this, FGameGRControl, false);
                FClientHandler.PlayReplay = true;
                FClientHandler.RecordReplay = FClientConfig.RecordReplay;

                // set game output events
                FGameGRControl.OpenGLStarted += new GRControl.DelegateOpenGLStarted(FClientHandler.OpenGLStarted);
                FGameGRControl.KeyDown += new KeyEventHandler(FClientHandler.KeyDown);
                FGameGRControl.KeyUp += new KeyEventHandler(FClientHandler.KeyUp);
                FGameGRControl.MouseDown += new MouseEventHandler(FClientHandler.MouseDown);
                FGameGRControl.MouseUp += new MouseEventHandler(FClientHandler.MouseUp);
                FGameGRControl.MouseMove += new MouseEventHandler(FClientHandler.MouseMove);
                FGameGRControl.MouseWheel += new MouseEventHandler(FClientHandler.MouseWheel);
                FGameGRControl.Paint += new PaintEventHandler(FClientHandler.Paint);
                FGameGRControl.KeyPress += new KeyPressEventHandler(FClientHandler.KeyPress);

                // start game
                FClientHandler.Active = true;
                ClientTimer.Enabled = true;

                FClientWait = 0;
            }
            else
            {
                ClientTimer.Enabled = false;
            }

            if (MainTabControl.SelectedTab == ServerTabPage)
            {
                Cursor.Show();
            }

            if (MainTabControl.SelectedTab == ServerBTabPage)
            {
                Cursor.Show();

                // kill old server engine
                if (FServerHandler != null)
                {
                    FServerHandler.Kill = true;
                    FServerHandler = null;
                }

                // create new server engine
                if (File.Exists(FClientConfig.MapFile))
                {
                    FServerHandler = new ServerHandler(FClientConfig.MapFile, FClientConfig.LocalGame);
                    FServerHandler.Active = true;

                    // kill old game enginge
                    if (FClientHandler != null)
                    {
                        FClientHandler.Kill = true; ;
                        FClientHandler = null;
                    }

                    // create new game engine
                    FClientHandler = new ClientHandler(this, FGameGRControl, false);
                    FClientHandler.PlayReplay = false;
                    FClientHandler.RecordReplay = FClientConfig.RecordReplay;

                    // set game output events
                    FGameGRControl.OpenGLStarted += new GRControl.DelegateOpenGLStarted(FClientHandler.OpenGLStarted);
                    FGameGRControl.KeyDown += new KeyEventHandler(FClientHandler.KeyDown);
                    FGameGRControl.KeyUp += new KeyEventHandler(FClientHandler.KeyUp);
                    FGameGRControl.MouseDown += new MouseEventHandler(FClientHandler.MouseDown);
                    FGameGRControl.MouseUp += new MouseEventHandler(FClientHandler.MouseUp);
                    FGameGRControl.MouseMove += new MouseEventHandler(FClientHandler.MouseMove);
                    FGameGRControl.MouseWheel += new MouseEventHandler(FClientHandler.MouseWheel);
                    FGameGRControl.Paint += new PaintEventHandler(FClientHandler.Paint);
                    FGameGRControl.KeyPress += new KeyPressEventHandler(FClientHandler.KeyPress);

                    // start game
                    FClientHandler.Active = true;
                    ServerTimer.Enabled = true;

                    FServerWait = 0;
                }
            }
            else
            {
                ServerTimer.Enabled = false;
            }

            if (MainTabControl.SelectedTab == InfoTabPage)
            {
                Cursor.Show();
            }

            // if current page is Game Page
            if (MainTabControl.SelectedTab == GameTabPage)
            {
                Cursor.Hide();

                GameTimer.Enabled = true;
            }
            else
            {
                GameTimer.Enabled = false;
            }

            if (MainTabControl.SelectedTab == HighscoreTabPage)
            {
                Cursor.Show();

                FHighscore = FClientConfig.SetHighscore(FDuration);

                int LMinutes = FDuration / 60000;
                int LSeconds = (FDuration % 60000) / 1000;
                PersonalLabel.Text = "Your Time: " + Convert.ToString(LMinutes) + ":" + Convert.ToString(LSeconds);

                LMinutes = FClientConfig.Highscore[0] / 60000;
                LSeconds = (FClientConfig.Highscore[0] % 60000) / 1000;
                Highscore0Label.Text = "Rank A) " + Convert.ToString(LMinutes) + ":" + Convert.ToString(LSeconds);

                LMinutes = FClientConfig.Highscore[1] / 60000;
                LSeconds = (FClientConfig.Highscore[1] % 60000) / 1000;
                Highscore1Label.Text = "Rank B) " + Convert.ToString(LMinutes) + ":" + Convert.ToString(LSeconds);

                LMinutes = FClientConfig.Highscore[2] / 60000;
                LSeconds = (FClientConfig.Highscore[2] % 60000) / 1000;
                Highscore2Label.Text = "Rank C) " + Convert.ToString(LMinutes) + ":" + Convert.ToString(LSeconds);

                LMinutes = FClientConfig.Highscore[3] / 60000;
                LSeconds = (FClientConfig.Highscore[3] % 60000) / 1000;
                Highscore3Label.Text = "Rank D) " + Convert.ToString(LMinutes) + ":" + Convert.ToString(LSeconds);

                LMinutes = FClientConfig.Highscore[4] / 60000;
                LSeconds = (FClientConfig.Highscore[4] % 60000) / 1000;
                Highscore4Label.Text = "Rank E) " + Convert.ToString(LMinutes) + ":" + Convert.ToString(LSeconds);

                Highscore0Label.Visible = true;
                Highscore1Label.Visible = true;
                Highscore2Label.Visible = true;
                Highscore3Label.Visible = true;
                Highscore4Label.Visible = true;

                HighscoreTimer.Enabled = true;
            }
            else
            {
                HighscoreTimer.Enabled = false;
            }
        }

        private void ExitLabel_MouseEnter(object sender, EventArgs e)
        {
            ExitLabel.Text = "EXIT";
        }

        private void ExitLabel_MouseLeave(object sender, EventArgs e)
        {
            ExitLabel.Text = "exit";
        }

        private void ExitLabel_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void InfoLabel_MouseEnter(object sender, EventArgs e)
        {
            InfoLabel.Text = "INFOS";
        }

        private void InfoLabel_MouseLeave(object sender, EventArgs e)
        {
            InfoLabel.Text = "infos";
        }

        private void InfoLabel_Click(object sender, EventArgs e)
        {
            MainTabControl.SelectedTab = InfoTabPage;
        }

        private void BackLabel_MouseEnter(object sender, EventArgs e)
        {
            BackLabel.Text = "BACK";
        }

        private void BackLabel_MouseLeave(object sender, EventArgs e)
        {
            BackLabel.Text = "back";
        }

        private void BackLabel_Click(object sender, EventArgs e)
        {
            MainTabControl.SelectedTab = MenuTabPage;
        }

        private void OptionsLabel_MouseEnter(object sender, EventArgs e)
        {
            OptionsLabel.Text = "OPTIONS";
        }

        private void OptionsLabel_MouseLeave(object sender, EventArgs e)
        {
            OptionsLabel.Text = "options";
        }

        private void OptionsLabel_Click(object sender, EventArgs e)
        {
            MainTabControl.SelectedTab = OptionsTabPage;
        }

        private void Back2Label_MouseEnter(object sender, EventArgs e)
        {
            Back2Label.Text = "BACK";
        }

        private void Back2Label_MouseLeave(object sender, EventArgs e)
        {
            Back2Label.Text = "back";
        }

        private void Back2Label_Click(object sender, EventArgs e)
        {
            MainTabControl.SelectedTab = MenuTabPage;
        }

        private void EnterLabel_MouseEnter(object sender, EventArgs e)
        {
            EnterLabel.Text = "ENTER GAME";
        }

        private void EnterLabel_MouseLeave(object sender, EventArgs e)
        {
            EnterLabel.Text = "enter game";
        }

        private void EnterLabel_Click(object sender, EventArgs e)
        {
            MainTabControl.SelectedTab = ClientTabPage;
        }

        private void CreateLabel_MouseEnter(object sender, EventArgs e)
        {
            CreateLabel.Text = "CREATE GAME";
        }

        private void CreateLabel_MouseLeave(object sender, EventArgs e)
        {
            CreateLabel.Text = "create game";
        }

        private void CreateLabel_Click(object sender, EventArgs e)
        {
            MainTabControl.SelectedTab = ServerTabPage;
        }

        private void ReplaysLabel_MouseEnter(object sender, EventArgs e)
        {
            ReplayLabel.Text = "RECORD REPLAYS";
        }

        private void ReplaysLabel_MouseLeave(object sender, EventArgs e)
        {
            ReplayLabel.Text = "record replays";
        }

        private void ReplaysLabel_Click(object sender, EventArgs e)
        {
            FClientConfig.RecordReplay = (!(FClientConfig.RecordReplay));

            if (FClientConfig.RecordReplay)
            {
                ReplaysValueLabel.Text = "on";
            }
            else
            {
                ReplaysValueLabel.Text = "off";
            }
        }

        private void TitleTimer_Tick(object sender, EventArgs e)
        {
            KeyLabel.Visible = (!(KeyLabel.Visible));
        }

        private void ClientTimer_Tick(object sender, EventArgs e)
        {
            if (FClientHandler != null)
            {
                if (MainTabControl.SelectedTab == ClientTabPage)
                {
                    ClientWaitLabel.Visible = (!(ClientWaitLabel.Visible));

                    if (FClientHandler.Acknowledged)
                    {
                        MainTabControl.SelectedTab = GameTabPage;
                    }
                    else
                    {
                        FClientWait++;
                        if (FClientWait > 60)
                        {
                            FClientHandler.Kill = true;
                            FClientHandler = null;

                            MainTabControl.SelectedTab = MenuTabPage;
                        }
                    }
                }
            }
        }

        private void LocalLabel_MouseEnter(object sender, EventArgs e)
        {
            LocalLabel.Text = "LOCAL GAME";
        }

        private void LocalLabel_MouseLeave(object sender, EventArgs e)
        {
            LocalLabel.Text = "local game";
        }

        private void RemoteLabel_MouseEnter(object sender, EventArgs e)
        {
            RemoteLabel.Text = "REMOTE GAME";
        }

        private void RemoteLabel_MouseLeave(object sender, EventArgs e)
        {
            RemoteLabel.Text = "remote game";
        }

        private void Back3label_MouseEnter(object sender, EventArgs e)
        {
            Back3label.Text = "BACK";
        }

        private void Back3label_MouseLeave(object sender, EventArgs e)
        {
            Back3label.Text = "back";
        }

        private void Back3label_Click(object sender, EventArgs e)
        {
            MainTabControl.SelectedTab = MenuTabPage;
        }

        private void RemoteLabel_Click(object sender, EventArgs e)
        {
            OpenFileDialog.InitialDirectory = System.Environment.CurrentDirectory;
            if (OpenFileDialog.ShowDialog() == DialogResult.OK)
            {
                FClientConfig.LocalGame = false;
                FClientConfig.MapFile = OpenFileDialog.FileName;

                MainTabControl.SelectedTab = ServerBTabPage;
            }
        }

        private void LocalLabel_Click(object sender, EventArgs e)
        {
            OpenFileDialog.InitialDirectory = System.Environment.CurrentDirectory;
            if (OpenFileDialog.ShowDialog() == DialogResult.OK)
            {
                FClientConfig.LocalGame = true;
                FClientConfig.MapFile = OpenFileDialog.FileName;

                MainTabControl.SelectedTab = ServerBTabPage;
            }
        }

        private void GameTimer_Tick(object sender, EventArgs e)
        {
            if (MainTabControl.SelectedTab == GameTabPage)
            {
                if (FClientHandler != null)
                {
                    if (FClientHandler.Ended || FClientHandler.Finished)
                    {
                        FDuration = FClientHandler.GameStop - FClientHandler.GameStart;

                        MainTabControl.SelectedTab = HighscoreTabPage;

                        FClientHandler.Active = false;
                        FClientHandler.Kill = true;
                        FClientHandler = null;

                        if (FServerHandler != null)
                        {
                            if (!(FServerHandler.GameRunning))
                            {
                                FServerHandler.Active = false;
                                FServerHandler.Kill = true;
                                FServerHandler = null;
                            }
                        }
                    }
                }
            }
        }

        private void HighscoreTimer_Tick(object sender, EventArgs e)
        {
            if (MainTabControl.SelectedTab == HighscoreTabPage)
            {
                switch (FHighscore)
                {
                    case 0:
                        {
                            Highscore0Label.Visible = (!(Highscore0Label.Visible));
                            break;
                        }

                    case 1:
                        {
                            Highscore1Label.Visible = (!(Highscore1Label.Visible));
                            break;
                        }

                    case 2:
                        {
                            Highscore2Label.Visible = (!(Highscore2Label.Visible));
                            break;
                        }

                    case 3:
                        {
                            Highscore3Label.Visible = (!(Highscore3Label.Visible));
                            break;
                        }

                    case 4:
                        {
                            Highscore4Label.Visible = (!(Highscore4Label.Visible));
                            break;
                        }
                }
            }
        }

        private void Back4Label_MouseEnter(object sender, EventArgs e)
        {
            Back4Label.Text = "BACK";
        }

        private void Back4Label_MouseLeave(object sender, EventArgs e)
        {
            Back4Label.Text = "back";
        }

        private void Back4Label_Click(object sender, EventArgs e)
        {
            MainTabControl.SelectedTab = MenuTabPage;
        }
    }
}