unit MainF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TCoordinate = record
    X : Single;
    Y : Single;
    Z : Single;
  end;

  TCube = record
    ID         : Byte;
    Pos        : TCoordinate;
    Rot        : TCoordinate;
    Scale      : TCoordinate;
    Multiplier : Single;
  end;

  TServerCube = record
    Cube     : TCube;
    RotSpeed : TCoordinate;
  end;

  TMap = record
    Map       : String[3];
    TimeStamp : LongInt;
    Length    : Byte;
    Cubes     : array of TServerCube;
  end;

  TMainForm = class(TForm)
    ListBox: TListBox;
    IDLabel: TLabel;
    IDEdit: TEdit;
    PosXLabel: TLabel;
    PosYLabel: TLabel;
    PosZLabel: TLabel;
    PosXEdit: TEdit;
    PosYEdit: TEdit;
    PosZEdit: TEdit;
    RotXLabel: TLabel;
    RotYLabel: TLabel;
    RotZLabel: TLabel;
    RotXEdit: TEdit;
    RotYEdit: TEdit;
    RotZEdit: TEdit;
    ScaleXLabel: TLabel;
    ScaleYLabel: TLabel;
    ScaleZLabel: TLabel;
    ScaleXEdit: TEdit;
    ScaleYEdit: TEdit;
    ScaleZEdit: TEdit;
    MultiplierLabel: TLabel;
    MultiplierEdit: TEdit;
    SpeedXLabel: TLabel;
    SpeedYLabel: TLabel;
    SpeedZLabel: TLabel;
    SpeedXEdit: TEdit;
    SpeedYEdit: TEdit;
    SpeedZEdit: TEdit;
    ButtonsBevel: TBevel;
    AddButton: TButton;
    DeleteButton: TButton;
    UpButton: TButton;
    DownButton: TButton;
    ActionsBevel: TBevel;
    DesignBevel: TBevel;
    LoadButton: TButton;
    SaveButton: TButton;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    NewButton: TButton;

    procedure LoadButtonClick(Sender: TObject);
    procedure NewButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ListBoxClick(Sender: TObject);
    procedure AddButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure UpButtonClick(Sender: TObject);
    procedure DownButtonClick(Sender: TObject);
  private
    { Private-Deklarationen }
    FCube : ^TServerCube;
    FMap  : TMap;

    procedure NewMap;
    procedure ShowMap;
  public
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.LoadButtonClick(Sender: TObject);
var
  LCorrupt    : Boolean;
  LFileHandle : THandle;
  LIndex      : LongInt;
  LServerCube : TServerCube;
begin
  if OpenDialog.Execute then
  begin
    NewMap;

    LFileHandle := FileOpen(OpenDialog.FileName, fmOpenRead or fmShareDenyNone);
    if (LFileHandle <> INVALID_HANDLE_VALUE) then
    begin
      try
        if (FileRead(LFileHandle, FMap.Map[1], Length(FMap.Map)) = Length(FMap.Map)) then
        begin
          if (FMap.Map = 'MAP') then
          begin
            if (FileRead(LFileHandle, FMap.TimeStamp, SizeOf(FMap.TimeStamp)) = SizeOf(FMap.TimeStamp)) then
            begin
              if (FMap.TimeStamp = 0) then
              begin
                if (FileRead(LFileHandle, FMap.Length, SizeOf(FMap.Length)) = SizeOf(FMap.Length)) then
                begin
                  if (FMap.Length > 0) then
                  begin
                    SetLength(FMap.Cubes, FMap.Length);

                    LCorrupt := false;
                    for LIndex := 1 to FMap.Length do
                    begin
                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.Cube.ID, SizeOf(LServerCube.Cube.ID)) <> SizeOf(LServerCube.Cube.ID));

                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.Cube.Pos.X, SizeOf(LServerCube.Cube.Pos.X)) <> SizeOf(LServerCube.Cube.Pos.X));
                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.Cube.Pos.Y, SizeOf(LServerCube.Cube.Pos.Y)) <> SizeOf(LServerCube.Cube.Pos.Y));
                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.Cube.Pos.Z, SizeOf(LServerCube.Cube.Pos.Z)) <> SizeOf(LServerCube.Cube.Pos.Z));

                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.Cube.Rot.X, SizeOf(LServerCube.Cube.Rot.X)) <> SizeOf(LServerCube.Cube.Rot.X));
                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.Cube.Rot.Y, SizeOf(LServerCube.Cube.Rot.Y)) <> SizeOf(LServerCube.Cube.Rot.Y));
                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.Cube.Rot.Z, SizeOf(LServerCube.Cube.Rot.Z)) <> SizeOf(LServerCube.Cube.Rot.Z));

                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.Cube.Scale.X, SizeOf(LServerCube.Cube.Scale.X)) <> SizeOf(LServerCube.Cube.Scale.X));
                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.Cube.Scale.Y, SizeOf(LServerCube.Cube.Scale.Y)) <> SizeOf(LServerCube.Cube.Scale.Y));
                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.Cube.Scale.Z, SizeOf(LServerCube.Cube.Scale.Z)) <> SizeOf(LServerCube.Cube.Scale.Z));

                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.Cube.Multiplier, SizeOf(LServerCube.Cube.Multiplier)) <> SizeOf(LServerCube.Cube.Multiplier));

                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.RotSpeed.X, SizeOf(LServerCube.Cube.Scale.X)) <> SizeOf(LServerCube.RotSpeed.X));
                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.RotSpeed.Y, SizeOf(LServerCube.Cube.Scale.Y)) <> SizeOf(LServerCube.RotSpeed.Y));
                      LCorrupt := LCorrupt or (FileRead(LFileHandle, LServerCube.RotSpeed.Z, SizeOf(LServerCube.Cube.Scale.Z)) <> SizeOf(LServerCube.RotSpeed.Z));

                      FMap.Cubes[Pred(LIndex)] := LServerCube;
                    end;

                    if LCorrupt then
                        ShowMessage('corrupt file - trying to show');
                  end;
                end
                else
                  ShowMessage('corrupt map file');
              end
              else
                ShowMessage('no map file');
            end
            else
              ShowMessage('corrupt map file');
          end
          else
            ShowMessage('no map file');
        end
        else
          ShowMessage('corrupt map file');
      finally
        FileClose(LFileHandle);
      end;
    end;

    ShowMap;
  end;
end;

procedure TMainForm.NewMap;
begin
  FMap.Map       := 'MAP';
  FMap.TimeStamp := 0;
  FMap.Length    := 0;
  SetLength(FMap.Cubes, 0);
end;

procedure TMainForm.ShowMap;
var
  LIndex : LongInt;
begin
  ListBox.Items.BeginUpdate;
  try
    ListBox.Items.Clear;

    for LIndex := 1 to Length(FMap.Cubes) do
    begin
      ListBox.Items.Add('Cube ' + IntToStr(Pred(LIndex)));

      if (LIndex = 1) then
        ListBox.Items[Pred(LIndex)] := ListBox.Items[Pred(LIndex)] + ' - START';
      if (LIndex = Length(FMap.Cubes)) then
        ListBox.Items[Pred(LIndex)] := ListBox.Items[Pred(LIndex)] + ' - ZIEL';
    end;
  finally
    ListBox.Items.EndUpdate;
  end;
end;

procedure TMainForm.UpButtonClick(Sender: TObject);
var
  LTemp : TServerCube;
begin
  if (ListBox.ItemIndex > 0) then
  begin
    ListBoxClick(nil);
    FCube := nil;

    LTemp := FMap.Cubes[Pred(ListBox.ItemIndex)];
    FMap.Cubes[Pred(ListBox.ItemIndex)] := FMap.Cubes[ListBox.ItemIndex];
    FMap.Cubes[ListBox.ItemIndex] := LTemp;

    ListBox.ItemIndex := Pred(ListBox.ItemIndex);
    ListBoxClick(nil);
  end;
end;

procedure TMainForm.NewButtonClick(Sender: TObject);
begin
  NewMap;
  ShowMap;
end;

procedure TMainForm.DeleteButtonClick(Sender: TObject);
var
  LIndex : LongInt;
  LItem  : LongInt;
begin
  ListBoxClick(nil);
  FCube := nil;

  LItem := ListBox.ItemIndex;

  if (LItem >= 0) and (LItem < FMap.Length) then
  begin
    for LIndex := LItem to (FMap.Length - 2) do
      FMap.Cubes[LIndex] := FMap.Cubes[Succ(LIndex)];

    FMap.Length := Pred(FMap.Length);
    SetLength(FMap.Cubes, FMap.Length);
  end;

  ShowMap;
  if (LItem < ListBox.Items.Count) then
    ListBox.ItemIndex := LItem
  else
    ListBox.ItemIndex := Pred(LItem);
  ListBoxClick(nil);
end;

procedure TMainForm.DownButtonClick(Sender: TObject);
var
  LTemp : TServerCube;
begin
  if (ListBox.Items.Count >= 2) and (ListBox.ItemIndex < Pred(ListBox.Items.Count)) then
  begin
    ListBoxClick(nil);
    FCube := nil;

    LTemp := FMap.Cubes[Succ(ListBox.ItemIndex)];
    FMap.Cubes[Succ(ListBox.ItemIndex)] := FMap.Cubes[ListBox.ItemIndex];
    FMap.Cubes[ListBox.ItemIndex] := LTemp;

    ListBox.ItemIndex := Succ(ListBox.ItemIndex);
    ListBoxClick(nil);
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  NewMap;
  ShowMap;

  FCube := nil;
end;

procedure TMainForm.ListBoxClick(Sender: TObject);
begin
  if (ListBox.ItemIndex >= 0) and (ListBox.ItemIndex < ListBox.Items.Count) then
  begin
    DownButton.Enabled := (ListBox.ItemIndex < Pred(ListBox.Items.Count));
    UpButton.Enabled   := (ListBox.ItemIndex > 0);

    if (FCube <> nil) then
    begin
      if not(TryStrToFloat(PosXEdit.Text, FCube^.Cube.Pos.X)) then
        raise Exception.Create('value could not be set - POS.X');
      if not(TryStrToFloat(PosYEdit.Text, FCube^.Cube.Pos.Y)) then
        raise Exception.Create('value could not be set - POS.Y');
      if not(TryStrToFloat(PosZEdit.Text, FCube^.Cube.Pos.Z)) then
        raise Exception.Create('value could not be set - POS.Z');

      if not(TryStrToFloat(RotXEdit.Text, FCube^.Cube.Rot.X)) then
        raise Exception.Create('value could not be set - ROT.X');
      if not(TryStrToFloat(RotYEdit.Text, FCube^.Cube.Rot.Y)) then
        raise Exception.Create('value could not be set - ROT.Y');
      if not(TryStrToFloat(RotZEdit.Text, FCube^.Cube.Rot.Z)) then
        raise Exception.Create('value could not be set - ROT.Z');

      if not(TryStrToFloat(ScaleXEdit.Text, FCube^.Cube.Scale.X)) then
        raise Exception.Create('value could not be set - SCL.X');
      if not(TryStrToFloat(ScaleYEdit.Text, FCube^.Cube.Scale.Y)) then
        raise Exception.Create('value could not be set - SCL.Y');
      if not(TryStrToFloat(ScaleZEdit.Text, FCube^.Cube.Scale.Z)) then
        raise Exception.Create('value could not be set - SCL.Z');

      if not(TryStrToFloat(SpeedXEdit.Text, FCube^.RotSpeed.X)) then
        raise Exception.Create('value could not be set - SPD.X');
      if not(TryStrToFloat(SpeedYEdit.Text, FCube^.RotSpeed.Y)) then
        raise Exception.Create('value could not be set - SPD.Y');
      if not(TryStrToFloat(SpeedZEdit.Text, FCube^.RotSpeed.Z)) then
        raise Exception.Create('value could not be set - SPD.Z');

      if not(TryStrToFloat(MultiplierEdit.Text, FCube^.Cube.Multiplier)) then
        raise Exception.Create('value could not be set - MULTI');
    end;

    FCube := @FMap.Cubes[ListBox.ItemIndex];

    if (FCube <> nil) then
    begin
      IDEdit.Text := IntToStr(ListBox.ItemIndex);

      PosXEdit.Text := FloatToStr(FCube^.Cube.Pos.X);
      PosYEdit.Text := FloatToStr(FCube^.Cube.Pos.Y);
      PosZEdit.Text := FloatToStr(FCube^.Cube.Pos.Z);

      RotXEdit.Text := FloatToStr(FCube^.Cube.Rot.X);
      RotYEdit.Text := FloatToStr(FCube^.Cube.Rot.Y);
      RotZEdit.Text := FloatToStr(FCube^.Cube.Rot.Z);

      ScaleXEdit.Text := FloatToStr(FCube^.Cube.Scale.X);
      ScaleYEdit.Text := FloatToStr(FCube^.Cube.Scale.Y);
      ScaleZEdit.Text := FloatToStr(FCube^.Cube.Scale.Z);

      SpeedXEdit.Text := FloatToStr(FCube^.RotSpeed.X);
      SpeedYEdit.Text := FloatToStr(FCube^.RotSpeed.Y);
      SpeedZEdit.Text := FloatToStr(FCube^.RotSpeed.Z);

      MultiplierEdit.Text := FloatToStr(FCube^.Cube.Multiplier);
    end;
  end;
end;

procedure TMainForm.AddButtonClick(Sender: TObject);
var
  LIndex : LongInt;
  LItem  : LongInt;
begin
  FMap.Length := Succ(FMap.Length);
  SetLength(FMap.Cubes, FMap.Length);

  ListBoxClick(nil);
  FCube := nil;

  LItem := ListBox.ItemIndex;
  ShowMap;
  ListBox.ItemIndex := LItem;

  if (LItem >= 0) and (LItem < FMap.Length) then
  begin
    for LIndex := (FMap.Length - 2) downto LItem do
      FMap.Cubes[Succ(LIndex)] := FMap.Cubes[LIndex];

    FMap.Cubes[LItem].Cube.Pos.X := 0;
    FMap.Cubes[LItem].Cube.Pos.Y := 0;
    FMap.Cubes[LItem].Cube.Pos.Z := 0;

    FMap.Cubes[LItem].Cube.Rot.X := 0;
    FMap.Cubes[LItem].Cube.Rot.Y := 0;
    FMap.Cubes[LItem].Cube.Rot.Z := 0;

    FMap.Cubes[LItem].Cube.Scale.X := 0;
    FMap.Cubes[LItem].Cube.Scale.Y := 0;
    FMap.Cubes[LItem].Cube.Scale.Z := 0;

    FMap.Cubes[LItem].RotSpeed.X := 0;
    FMap.Cubes[LItem].RotSpeed.Y := 0;
    FMap.Cubes[LItem].RotSpeed.Z := 0;

    FMap.Cubes[LItem].Cube.Multiplier := 0;
  end;

  ListBoxClick(nil);
end;

procedure TMainForm.SaveButtonClick(Sender: TObject);
var
  LCorrupt    : Boolean;
  LFileHandle : THandle;
  LIndex      : LongInt;
  LServerCube : TServerCube;
begin
  if SaveDialog.Execute then
  begin
    ListBoxClick(nil);

    LFileHandle := FileCreate(SaveDialog.FileName);
    if (LFileHandle <> INVALID_HANDLE_VALUE) then
    begin
      try
        if (FileWrite(LFileHandle, FMap.Map[1], Length(FMap.Map)) = Length(FMap.Map)) then
        begin
          if (FileWrite(LFileHandle, FMap.TimeStamp, SizeOf(FMap.TimeStamp)) = SizeOf(FMap.TimeStamp)) then
          begin
            if (FileWrite(LFileHandle, FMap.Length, SizeOf(FMap.Length)) = SizeOf(FMap.Length)) then
            begin
              LCorrupt := false;
              for LIndex := 1 to FMap.Length do
              begin
                LServerCube := FMap.Cubes[Pred(LIndex)];
                LServerCube.Cube.ID := Pred(LIndex);

                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.Cube.ID, SizeOf(LServerCube.Cube.ID)) <> SizeOf(LServerCube.Cube.ID));

                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.Cube.Pos.X, SizeOf(LServerCube.Cube.Pos.X)) <> SizeOf(LServerCube.Cube.Pos.X));
                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.Cube.Pos.Y, SizeOf(LServerCube.Cube.Pos.Y)) <> SizeOf(LServerCube.Cube.Pos.Y));
                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.Cube.Pos.Z, SizeOf(LServerCube.Cube.Pos.Z)) <> SizeOf(LServerCube.Cube.Pos.Z));

                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.Cube.Rot.X, SizeOf(LServerCube.Cube.Rot.X)) <> SizeOf(LServerCube.Cube.Rot.X));
                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.Cube.Rot.Y, SizeOf(LServerCube.Cube.Rot.Y)) <> SizeOf(LServerCube.Cube.Rot.Y));
                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.Cube.Rot.Z, SizeOf(LServerCube.Cube.Rot.Z)) <> SizeOf(LServerCube.Cube.Rot.Z));

                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.Cube.Scale.X, SizeOf(LServerCube.Cube.Scale.X)) <> SizeOf(LServerCube.Cube.Scale.X));
                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.Cube.Scale.Y, SizeOf(LServerCube.Cube.Scale.Y)) <> SizeOf(LServerCube.Cube.Scale.Y));
                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.Cube.Scale.Z, SizeOf(LServerCube.Cube.Scale.Z)) <> SizeOf(LServerCube.Cube.Scale.Z));

                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.Cube.Multiplier, SizeOf(LServerCube.Cube.Multiplier)) <> SizeOf(LServerCube.Cube.Multiplier));

                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.RotSpeed.X, SizeOf(LServerCube.Cube.Scale.X)) <> SizeOf(LServerCube.RotSpeed.X));
                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.RotSpeed.Y, SizeOf(LServerCube.Cube.Scale.Y)) <> SizeOf(LServerCube.RotSpeed.Y));
                LCorrupt := LCorrupt or (FileWrite(LFileHandle, LServerCube.RotSpeed.Z, SizeOf(LServerCube.Cube.Scale.Z)) <> SizeOf(LServerCube.RotSpeed.Z));
              end;

              if LCorrupt then
                ShowMessage('could not write - entry');
            end
            else
              ShowMessage('could not write - length');
          end
          else
            ShowMessage('could not write - timestamp');
        end
        else
          ShowMessage('could not write - header');
      finally
        FileClose(LFileHandle);
      end;
    end;
  end;
end;

end.

